<?php
/*
 Plugin Name: Counterize
 Plugin URI: http://www.gabsoftware.com/products/scripts/counterize/
 Description: Simple counter-plugin with no external libs - saves timestamps, visited URL, referring URL, operating systems and browser informations in database, and can display total hits, unique hits and other statistics in WordPress webpages. It is based on Counterize II by Steffen Forkmann (http://www.navision-blog.de/counterize). Admin-interface available with detailed information...
 Version: 3.0.16
 Author: Gabriel Hautclocq
 Author URI: http://www.gabsoftware.com/
 License: ISC
*/

//error_reporting(E_ALL);

// security check
if ( !defined( 'WP_PLUGIN_DIR') )
{
	die("There is nothing to see here!");
}

/* global variables */
$counterize_plugin_dir = WP_PLUGIN_DIR . '/' . plugin_basename(dirname(__FILE__));
$counterize_plugin_url = WP_PLUGIN_URL . '/' . plugin_basename(dirname(__FILE__));

$counterize_MajorVersion	= 3;
$counterize_MinorVersion	= 0;
$counterize_Revision		= 16;

require_once("browsniff.php");

// Counterize Tables
function counterize_agentsTable()
{
	global $wpdb;
	return $wpdb->prefix . "Counterize_UserAgents";
}

function counterize_logTable()
{
	global $wpdb;
	return $wpdb->prefix . "Counterize";
}

function counterize_pageTable()
{
	global $wpdb;
	return $wpdb->prefix . "Counterize_Pages";
}

function counterize_refererTable()
{
	global $wpdb;
	return $wpdb->prefix . "Counterize_Referers";
}

function counterize_keywordTable()
{
	global $wpdb;
	return $wpdb->prefix . "Counterize_Keywords";
}

// Returns the value of the specified option
function counterize_get_option($name)
{
	$options = get_option('counterize_options');
	if ( isset( $options[$name] ) )
	{
		return $options[$name];
	}
	else
	{
		return FALSE;
	}
}

// Sets the value of the specified option
function counterize_set_option($name, $value)
{
	$options = get_option('counterize_options');
	$options[$name] = $value;

	return update_option('counterize_options', $options );
}

function counterize_get_version($what = 'all')
{
	switch( $what )
	{
		case 'all':
			return get_option('counterize_version');
			break;
		case 'major':
			$version_array = explode('.', get_option('counterize_version'));
			return $version_array[0];
			break;
		case 'minor':
			$version_array = explode('.', get_option('counterize_version'));
			return $version_array[1];
			break;
		case 'revision':
			$version_array = explode('.', get_option('counterize_version'));
			return $version_array[2];
			break;
		default:
			return get_option('counterize_version');
	}
}

// seed with microseconds
function make_seed()
{
  list($usec, $sec) = explode(' ', microtime());
  return (float) $sec + ((float) $usec * 100000);
}

function get_random_number($min, $max)
{
	mt_srand(make_seed());
	return mt_rand($min, $max);
}

// Returns how many entries there are in the DB.
function counterize_getamount($only_this_month = false)
{
	global $wpdb;
	$sql = 'SELECT COUNT(1) FROM ' . counterize_logTable();
	if($only_this_month)
	{
		$sql .= " WHERE timestamp >= '" . date("Y-m-01") . "'";
	}
	return $wpdb->get_var($sql);
}

// Returns how many key-words there are in the DB.
function counterize_getkeywordamount()
{
	global $wpdb;
	$sql = "SELECT SUM(count) FROM " . counterize_keywordTable() . " WHERE keywordID <> 1";
	return $wpdb->get_var($sql);
}

// Return how many unique entries there are in the DB.
function counterize_getuniqueamount()
{
	global $wpdb;
	$sql = "SELECT COUNT(DISTINCT haship) "
		. " FROM " . counterize_logTable();
	return $wpdb->get_var($sql);
}

// Returns amount of entries in the DB matching the visiting IP-address.
function counterize_getfromcurrentip()
{
	global $wpdb;
	$sql = "SELECT COUNT(1) "
		. " FROM " . counterize_logTable()
		. " WHERE IP = '" . $_SERVER['REMOTE_ADDR'] . "'";
	return $wpdb->get_var($sql);
}

// Returns amount of hits today.
function counterize_gethitstoday()
{
	global $wpdb;
	$today = date("Y-m-d");
	$sql = "SELECT COUNT(1) FROM " . counterize_logTable() . " WHERE timestamp >= '{$today}'";
	return $wpdb->get_var($sql);
}

// Returns amount of hits during the last 7 days.
function counterize_getlatest7days()
{
	global $wpdb;
	$sevendaysago = date("Y-m-d", time() - 604800); //604800 = 86400 * 7
	$sql = "SELECT COUNT(1) FROM " . counterize_logTable() . " WHERE timestamp >= '{$sevendaysago}'";
	return $wpdb->get_var($sql);
}

// From Curtis(http://www.graymattersonline.net/)
// Returns amount of unique IP's in the last 7 days
function counterize_getuniquelatest7days()
{
	global $wpdb;
	$sevendaysago = date("Y-m-d", time() - 604800); //604800 = 86400 * 7
	$sql = "SELECT COUNT(DISTINCT haship) "
		. " FROM " . counterize_logTable()
		. " WHERE timestamp >= '{$sevendaysago}'";
	return $wpdb->get_var($sql);
}

// Returns the number of currently online users
function counterize_get_online_users()
{
	global $wpdb;
	$timestamp = gmdate("Y-m-d H:i:s", time() + (get_option('gmt_offset') * 3600 )); //3600 = 60 * 60
	$sql = "SELECT COUNT(DISTINCT haship) "
		. " FROM " . counterize_logTable()
		. " WHERE timestamp > DATE_SUB('{$timestamp}', INTERVAL 5 MINUTE)";
	return $wpdb->get_var($sql);
}

// Returns amount of unique referer-URl's
function counterize_getuniquereferers()
{
	global $wpdb;
	$sql = "SELECT COUNT(DISTINCT refererID) "
		. " FROM " . counterize_logTable();
	return $wpdb->get_var($sql);
}

// Returns amount of unique browser-strings in DB.
function counterize_getuniquebrowsers()
{
	global $wpdb;
	$sql = "SELECT COUNT(1) "
		. " FROM ". counterize_agentsTable();
	return $wpdb->get_var($sql);
}

// Returns amount on current article
function counterize_getHitsOnCurrentArticle()
{
	global $wpdb;
	if ($_SERVER['REQUEST_URI']){
		$requesturl = $wpdb->escape($_SERVER['REQUEST_URI']);
		$sql = "SELECT count FROM " . counterize_pageTable() . " WHERE url = '{$requesturl}'";
		return $wpdb->get_var($sql);
	}
	return 0;
}

// Returns amount of unique URl's
function counterize_getuniqueurl()
{
	global $wpdb;
	$sql = 'SELECT COUNT(1) FROM ' . counterize_refererTable();
	return $wpdb->get_var($sql);
}

function counterize_return_first_hit($dateformat = "j/n-Y")
{
	global $wpdb;
	$sql = "SELECT MIN(timestamp) "
		. " FROM " . counterize_logTable();
	$t = $wpdb->get_var($sql);
	return date($dateformat, strtotime($t));
}

// show the most visited pages
function counterize_most_visited_pages($number = 10, $width = 300)
{
	global $wpdb;
	$number = $wpdb->escape($number);
	$sql = "SELECT count AS amount , url AS url, url AS label, url AS label2 "
		. " FROM " . counterize_pageTable()
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);
	counterize_renderstats_vertical($rows, __('Pages', 'counterize'), $width, false);
}

function counterize_most_visited_pages24hrs($number = 10, $width = 300)
{
	global $wpdb;
	$onedayago = date("Y-m-d H:i:s", time() - 86400);
	$sql = "SELECT COUNT(m.id) AS amount, p.url AS url, p.url AS label, p.url AS label2 "
		. " FROM " . counterize_logTable() . " m, " . counterize_pageTable() . " p "
		. " WHERE m.pageID = p.pageID"
		. " AND m.timestamp >= '{$onedayago}'"
		. " GROUP BY p.url "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);
	counterize_renderstats_vertical($rows, __('Pages', 'counterize'), $width);
}

function counterize_most_visited_referrers($number = 10, $width = 300)
{
	global $wpdb;
	$sql = "SELECT count AS amount, name AS label, name AS label2, name AS url FROM ".counterize_refererTable()
		. " WHERE name <> 'unknown' "
		. " AND name NOT LIKE '" . $wpdb->escape(get_option("home")) . "%%' "
		. " AND name NOT LIKE '" . $wpdb->escape(get_option("siteurl")) . "%%' "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);
	counterize_renderstats_vertical($rows, __('Referers', 'counterize'), $width);
}

function counterize_most_visited_referrers24hrs($number = 10, $width = 300)
{
	global $wpdb;
	$onedayago = date("Y-m-d H:i:s", time() - 86400);
	$sql = "SELECT COUNT(m.id) AS amount, r.name AS label, r.name AS label2, r.name AS url "
		. " FROM " . counterize_logTable() . " m, ". counterize_refererTable() . " r "
		. " WHERE m.refererID = r.refererID "
		. " AND r.name <> 'unknown' "
		. " AND r.name NOT LIKE '" . $wpdb->escape(get_option("home")) . "%%' "
		. " AND r.name NOT LIKE '" . $wpdb->escape(get_option("siteurl")) . "%%' "
		. " AND m.timestamp >= '{$onedayago}'"
		. " GROUP BY r.name "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);
	counterize_renderstats_vertical($rows, __('Referers', 'counterize'), $width);
}

function counterize_most_visited_ips($number = 10, $width = 300)
{
	global $wpdb;
	$number = $wpdb->escape($number);
	$sql = "SELECT COUNT(IP) AS amount, IP AS label, CONCAT('http://whatismyipaddress.com/ip/', IP) AS url "
		. " FROM " . counterize_logTable()
		. " WHERE IP <> 'unavailable' "
		. " GROUP BY IP "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);

	counterize_renderstats_vertical($rows, __('IP', 'counterize'), $width);
}

function counterize_most_visited_ips24hrs($number = 10, $width = 300)
{
	global $wpdb;
	$onedayago = date("Y-m-d H:i:s", time() - 86400);
	$number = $wpdb->escape($number);
	$sql = "SELECT COUNT(IP) AS amount, IP AS label, CONCAT('http://whatismyipaddress.com/ip/', IP) AS url "
		. " FROM " . counterize_logTable()
		. " WHERE IP <> 'unavailable' "
		. " AND timestamp >= '{$onedayago}'"
		. " GROUP BY IP "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);

	counterize_renderstats_vertical($rows, __('IP', 'counterize'), $width);
}

function counterize_most_searched_keywords($number = 10, $width = 300)
{
	global $wpdb;
	$number = $wpdb->escape($number);
	$sql = "SELECT count AS amount, keyword AS label, keyword AS label2 "
		. " FROM " . counterize_keywordTable()
		. " WHERE keywordID <> 1 "
		. " GROUP BY keyword "
		. " ORDER BY count DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);
	counterize_renderstats_vertical($rows, __('Keywords', 'counterize'), $width);
}

function counterize_most_searched_keywords_today($number = 10, $width = 300)
{
	global $wpdb;
	$today = date("Y-m-d");
	$number = $wpdb->escape($number);
	$sql = "SELECT COUNT(1) AS amount, k.keyword AS label, k.keyword AS label2 "
		. " FROM " . counterize_keywordTable() . " k, " . counterize_logTable() . " l, " . counterize_refererTable() . " r "
		. " WHERE r.refererID = l.refererID "
		. " AND r.keywordID = k.keywordID "
		. " AND k.keywordID <> 1 "
		. " AND l.timestamp >= '{$today}' "
		. " GROUP BY k.keyword "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";

	$rows = $wpdb->get_results($sql);

	counterize_renderstats_vertical($rows, __('Keywords', 'counterize'), $width);
}

function counterize_most_used_browsers_without_version($number = 10, $width = 300)
{
	global $wpdb;
	$number = $wpdb->escape($number);
	$sql = "SELECT SUM(count) AS amount, browserName AS label, browserName AS label2, browserCode, browserURL AS url "
		. " FROM " . counterize_agentsTable()
		. " GROUP BY label "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);

	reset($rows);
	while (list($i, $r) = each($rows))
	{
		$row =& $rows[$i];
		if($row->label == " " || $row->label == "")
		{
			$row->label = __("unknown", "counterize");
		}
		else
		{
			$row->label = counterize_get_image_url ($row->browserCode, $row->label, $row->url) . " " . $row->label;
		}
	}

	counterize_renderstats_vertical($rows, __('Browsers', 'counterize'), $width, true, "100%", false);
}




function counterize_most_used_browsers_collapsible($nb_parent_items = 10, $nb_child_items = 15, $width = 300)
{
	global $wpdb;

	$nb_parent_items = (int) $wpdb->escape($nb_parent_items);
	$nb_child_items  = (int) $wpdb->escape($nb_child_items);

	$sql_parents =
			"SELECT browserName AS label, browserCode AS code, browserURL as url, SUM( count ) AS amount "
		. "  FROM " . counterize_agentsTable()
		. "  GROUP BY label "
		. "  ORDER BY amount DESC , label ASC "
		. "  LIMIT {$nb_parent_items} ";

	$sql_child =
			"SELECT browserVersion AS label, SUM( count ) AS amount "
		. "  FROM " . counterize_agentsTable()
		. "  WHERE browserName = %s "
		. "  GROUP BY label "
		. "  ORDER BY amount DESC , label ASC "
		. "  LIMIT %d";

	$rows_parents = $wpdb->get_results($sql_parents);

	$rows_child = Array();

	$counter = 0;
	foreach ($rows_parents as $parent)
	{
		$sql = $wpdb->prepare($sql_child, $parent->label, $nb_child_items);
		$rows_child[$counter] = $wpdb->get_results($sql);

		if($parent->label == " " || $parent->label == "")
		{
			$parent->label = __("unknown", "counterize");
		}
		else
		{
			$parent->label = counterize_get_image_url ($parent->code, $parent->label, $parent->url) . " " . $parent->label;
		}

		$counter++;
	}
	$random = get_random_number(1000000, 1999999);
	counterize_renderstats_vertical_collapsed($rows_parents, $rows_child, $random, __('Browsers', 'counterize'), $width, true, "100%", false);
}

function counterize_most_used_browsers($number = 10, $width = 300)
{
	global $wpdb;
	$number = $wpdb->escape($number);
	$sql = "SELECT "
		. " 	SUM(count) AS amount, "
		. " 	CONCAT(CONCAT(browserName,' '),browserVersion) AS label, "
		. " 	CONCAT(CONCAT(browserName,' '),browserVersion) AS label2, "
		. " 	browserCode, "
		. " 	browserURL AS url "
		. " FROM " . counterize_agentsTable()
		. " GROUP BY label "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";
	$rows = $wpdb->get_results($sql);

	reset($rows);
	while (list($i, $r) = each($rows))
	{
		$row =& $rows[$i];
		if($row->label == " " || $row->label == "")
		{
			$row->label = __("unknown", "counterize");
		}
		else
		{
			$row->label = counterize_get_image_url ($row->browserCode, $row->label, $row->url) . " " . $row->label;
		}
	}

	counterize_renderstats_vertical($rows, __('Browsers', 'counterize'), $width, true, "100%", false);
}

function counterize_most_used_os_without_version($number = 10, $width = 300)
{
	global $wpdb;
	$number = $wpdb->escape($number);
	$sql = "SELECT SUM(count) AS amount, osName AS label, osName AS label2, osCode, osURL AS url "
		. " FROM " . counterize_agentsTable()
		. " GROUP BY label "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";

	$rows = $wpdb->get_results($sql);

	reset($rows);
	while (list($i, $r) = each($rows))
	{
		$row =& $rows[$i];
		if($row->label == " " || $row->label == "")
		{
			$row->label = __("unknown","counterize");
		}
		else
		{
			$row->label = counterize_get_image_url ($row->osCode, $row->label, $row->url) . " " . $row->label;
		}
	}

	counterize_renderstats_vertical($rows, __('Operating systems', 'counterize'), $width, true, "100%", false);
}

function counterize_most_used_os_collapsible($nb_parent_items = 10, $nb_child_items = 15, $width = 300)
{
	global $wpdb;

	$nb_parent_items = (int) $wpdb->escape($nb_parent_items);
	$nb_child_items  = (int) $wpdb->escape($nb_child_items);

	$sql_parents =
			"SELECT osName AS label, osCode AS code, osURL as url, SUM( count ) AS amount "
		. "  FROM " . counterize_agentsTable()
		. "  GROUP BY label "
		. "  ORDER BY amount DESC , label ASC "
		. "  LIMIT {$nb_parent_items} ";

	$sql_child =
			"SELECT osVersion AS label, SUM( count ) AS amount "
		. "  FROM " . counterize_agentsTable()
		. "  WHERE osName = %s "
		. "  GROUP BY label "
		. "  ORDER BY amount DESC , label ASC "
		. "  LIMIT %d";

	$rows_parents = $wpdb->get_results($sql_parents);

	$rows_child = Array();

	$counter = 0;
	foreach ($rows_parents as $parent)
	{
		$sql = $wpdb->prepare($sql_child, $parent->label, $nb_child_items);
		$rows_child[$counter] = $wpdb->get_results($sql);

		if($parent->label == " " || $parent->label == "")
		{
			$parent->label = __("unknown", "counterize");
		}
		else
		{
			$parent->label = counterize_get_image_url ($parent->code, $parent->label, $parent->url) . " " . $parent->label;
		}

		$counter++;
	}
	$random = get_random_number(2000000, 2999999);
	counterize_renderstats_vertical_collapsed($rows_parents, $rows_child, $random, __('Operating systems', 'counterize'), $width, true, "100%", false);
}

function counterize_most_used_os($number = 10, $width = 300)
{
	global $wpdb;
	$number = $wpdb->escape($number);
	$sql = "SELECT SUM(count) AS amount, CONCAT(CONCAT(osName,' '),osVersion) AS label, CONCAT(CONCAT(osName,' '),osVersion) AS label2, osCode, osURL AS url "
		. " FROM " . counterize_agentsTable()
		. " GROUP BY label "
		. " ORDER BY amount DESC "
		. " LIMIT {$number}";

	$rows = $wpdb->get_results($sql);

	reset($rows);
	while (list($i, $r) = each($rows))
	{
		$row =& $rows[$i];
		if($row->label == " " || $row->label == "")
		{
			$row->label = __("unknown","counterize");
		}
		else
		{
			$row->label = counterize_get_image_url ($row->osCode, $row->label, $row->url) . " " . $row->label;
		}
	}

	counterize_renderstats_vertical($rows, __('Operating systems', 'counterize'), $width, true, "100%", false);
}


function counterize_get_hits()
{
	?>
		<table width="100%" cellpadding="3" cellspacing="3">
		<tr>
			<td scope="col" style="width: 15%" align="center"><?php _e("Total hits: ", 'counterize'); ?>
			</td>
			<td scope="col" style="width: 15%" align="center"><?php _e("Hits from unique IPs: ", 'counterize'); ?>
			</td>
			<td scope="col" style="width: 15%" align="center"><?php _e("Total hits, today: ", 'counterize'); ?>
			</td>
			<td scope="col" style="width: 20%" align="center"><?php _e("Hits from unique IPs, today: ", 'counterize'); ?>
			</td>
			<td scope="col" style="width: 15%" align="center"><?php _e("Hits, the last 7 days: ", 'counterize'); ?>
			</td>
			<td scope="col" style="width: 20%" align="center"><?php _e("Unique hits, the last 7 days: ", 'counterize'); ?>
			</td>
		</tr>
		<tr>
			<td align="center"><strong><?php echo counterize_getamount(); ?></strong></td>
			<td align="center"><strong><?php echo counterize_getuniqueamount(); ?></strong></td>
			<td align="center"><strong><?php echo counterize_gethitstoday(); ?></strong></td>
			<td align="center"><strong><?php echo counterize_getuniquehitstoday(); ?></strong></td>
			<td align="center"><strong><?php echo counterize_getlatest7days(); ?></strong></td>
			<td align="center"><strong><?php echo counterize_getuniquelatest7days(); ?></strong></td>
		</tr>
	</table>

	<?php
}


function counterize_getdailystats($only_this_month = false)
{
	global $wpdb;

	$sql = "SELECT "
		. " 	DAYOFMONTH(timestamp) AS label, "
		. " 	DAYOFMONTH(timestamp) AS label2, "
		. " 	COUNT(1) AS amount "
		. " FROM " . counterize_logTable();

	if($only_this_month)
	{
		$sql .= " WHERE timestamp >= '" . date("Y-m-01") . "'";
	}
	$sql .= " GROUP BY label";
	return $wpdb->get_results($sql);
}

function counterize_getmonthlystats()
{
	global $wpdb;

	$sql = "SELECT "
		. " 	CONCAT(CONCAT(SUBSTRING(monthname(timestamp),1,3),' '),SUBSTRING(YEAR(timestamp),3,2)) AS label, "
		. " 	CONCAT(CONCAT(SUBSTRING(monthname(timestamp),1,3),' '),SUBSTRING(YEAR(timestamp),3,2)) AS label2, "
		. " 	COUNT(1) AS amount, "
		. " 	MONTH(timestamp) AS m, "
		. " 	YEAR(timestamp) AS y "
		. " FROM " . counterize_logTable()
		. " GROUP BY label "
		. " ORDER BY y,m";

	return $wpdb->get_results($sql);
}

function counterize_getweeklystats()
{
	global $wpdb;
	$sql = "SELECT "
		. " 	DAYNAME(timestamp) AS label, "
		. " 	DAYNAME(timestamp) AS label2, "
		. " 	COUNT(1) AS amount, "
		. " 	DAYOFWEEK(timestamp) AS day "
		. " FROM " . counterize_logTable()
		. " GROUP BY label "
		. " ORDER BY day";
	return $wpdb->get_results($sql);
}

// This function is still not done - needs more work...
function counterize_gethourlystats($hour = "undef", $type = "both")
{
	global $wpdb;

	$sql = "SELECT "
		. " 	HOUR(timestamp) AS label, "
		. " 	HOUR(timestamp) AS label2, "
		. " 	COUNT(1) AS amount "
		. " FROM " . counterize_logTable();

	$sql .= " GROUP BY label";
	return $wpdb->get_results($sql);
}

function counterize_ref_analyzer($referer)
{
	if ( !empty( $referer ) && $referer != "unknown" )
	{
		$domain = explode('/', $referer);

		$source = array(
			array('google','q'),
			array('alltheweb','query'),
			array('altavista','q'),
			array('aol','query'),
			array('excite','search'),
			array('hotbot','query'),
			array('lycos','query'),
			array('yahoo','p'),
			array('live','q'),
			array('t-online','q'),
			array('msn','q'),
			array('netscape','search')
		);

		$keyword = "";
		for($i=0; $i<count($source); $i++)
		{
			if(eregi($source[$i][0], $referer))
			{
				$parse = parse_url($referer);
				parse_str($parse['query'], $output);
				$keyword = $output[$source[$i][1]];
				break;
			}
		}

		return array('domain' => str_replace('www.', '', $domain[2]), 'keyword' => strtolower($keyword));
	}
	else
	{
		return array('domain' => '', 'keyword' => '');
	}
}

// Returns amount of unique hits today
function counterize_getuniquehitstoday()
{
	global $wpdb;
	$today = date("Y-m-d");
	$sql = "SELECT COUNT(DISTINCT haship) "
		. " FROM " . counterize_logTable()
		. " WHERE timestamp >= '{$today}'";
	return $wpdb->get_var($sql);
}

// Returns amount of hits today, from the visiting IP
function counterize_gethitstodayfromcurrentip()
{
	global $wpdb;
	$today = date("Y-m-d");
	$sql = "SELECT COUNT(1) "
		. " FROM " . counterize_logTable()
		. " WHERE timestamp >= '{$today}' "
		. " AND IP = '" . $_SERVER['REMOTE_ADDR'] . "'";
	return $wpdb->get_var($sql);
}

// Fetch information matching ID in DB.
function counterize_getentries($amount = 50, $entryID = null)
{
	global $wpdb;
	$limit = 50;
	$sql = "SELECT id, IP, haship, timestamp, p.url AS url, r.name AS referer, ua.name AS useragent, "
		. " m.refererID, m.agentID, m.pageID, k.keyword, k.keywordID "
		. " FROM "
		.	counterize_logTable() . " m, "
		.	counterize_pageTable() . " p, "
		.	counterize_agentsTable() . " ua, "
		.	counterize_refererTable() . " r, "
		.	counterize_keywordTable() . " k "
		. " WHERE m.pageID = p.pageID "
		. " AND m.agentID = ua.agentID "
		. " AND m.refererID = r.refererID "
		. " AND k.keywordID = r.keywordID";

	if( isset( $_GET["ipfilter"] ) && check_admin_referer( "action_ip_filter" ) )
	{
		$sql .= " AND m.IP = '" . $wpdb->escape($_GET["ipfilter"]) ."' ";
	}
	if( isset( $_GET["urifilter"] ) && check_admin_referer( "action_uri_filter" ) )
	{
		$sql .= " AND p.url = '" . $wpdb->escape($_GET["urifilter"]) . "' ";
	}
	if( isset( $_GET["refererfilter"] ) && check_admin_referer( "action_referer_filter" ) )
	{
		$sql .= " AND r.name = '" . $wpdb->escape($_GET["refererfilter"]) . "' ";
	}
	if( isset( $_GET["agentfilter"] ) && check_admin_referer( "action_agent_filter" ) )
	{
		$sql .= " AND ua.name = '" . $wpdb->escape($_GET["agentfilter"]) . "' ";
	}

	if(isset($entryID))
	{
		$sql .= " AND m.id = " . $wpdb->escape($entryID);
	}

	$sql .= " ORDER BY m.timestamp DESC";

	if ($amount == "")
	{
		$sql .= " LIMIT {$limit}";
	}
	else
	{
		if ( is_numeric($amount))
		{
			if ($amount > 0)
			{
				$sql .= " LIMIT {$amount}";
			}
			/* The user choose to display all the entries. This can lead to an 'out of memory' error, but he is warned. */
			/*else
			{
				$sql .= " LIMIT 50";
			}*/
		}
		else
		{
			$sql .= " LIMIT {$limit}";
		}
	}

	$res = $wpdb->get_results($sql);

	return $res;
}

// Append information to DB
function counterize_add_callback()
{
	global $wpdb;
	global $user_ID;
	global $counterize_plugin_dir;

	// Set to unknown, if we're unable to extract information below.
	$referer = $remoteaddr = $useragent = $requesturl = 'unknown';

	if (isset( $_SERVER['REMOTE_ADDR'] ) )
	{
		$remoteaddr = $_SERVER['REMOTE_ADDR'];
	}
	if (isset( $_SERVER['HTTP_USER_AGENT'] ) )
	{
		$useragent = $_SERVER['HTTP_USER_AGENT'];
	}
	if (isset( $_SERVER['REQUEST_URI'] ) )
	{
		$requesturl = $_SERVER['REQUEST_URI'];
	}
	if (isset( $_SERVER['HTTP_REFERER'] ) )
	{
		$referer = $_SERVER['HTTP_REFERER'];
	}

	$this_url = "http://" . $_SERVER['HTTP_HOST'] . $requesturl;
	// Check to see if we really want to insert the entry...
	$checkval = 0;

	//
	// Bots detected and excluded
	//
	$botarray = Array();
	if (file_exists($counterize_plugin_dir . "/botlist.txt"))
	{
		$botarray = file($counterize_plugin_dir . "/botlist.txt", FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
	}

	// Run through bot-array and see if there's anything we don't like...
	foreach ($botarray as $entry)
	{
		if (stristr($useragent, $entry))
		{
			$checkval = 1;
			break;
		}
	}

	// From SHRIKEE, don't count RSS and other stuff...
	// Exclude files from being counted
	if ($checkval == 0)
	{
		// Exclude if referer is the same as url
		if ($this_url==$referer )
		{
			$checkval = 1;
		}

		// Exclude RSS feeds (Both with and without permalinks)
		// Stating just feed would make it impossible to name a page or post 'feed'
		/* if (stristr($requesturl, "feed/"))
		$checkval = 1;
		if (stristr($requesturl, "feed="))
		$checkval = 1;
		*/
		// Exclude files which browsers like safari and opera request on each page
		if (stristr($requesturl, "robots.txt"))
		{
			$checkval = 1;
		}
		elseif (stristr($requesturl, "favicon.ico"))
		{
			$checkval = 1;
		}

		// Exclude any admin or core files
		elseif (stristr($requesturl, "wp-includes/"))
		{
			$checkval = 1;
		}
		elseif (stristr($requesturl, "wp-admin/"))
		{
			$checkval = 1;
		}
		elseif (stristr($requesturl, "wp-content/"))
		{
			$checkval = 1;
		}
	}

	// more extensions?
	elseif (stristr($requesturl, ".jpg"))
	{
		$checkval = 1;
	}
	elseif (stristr($requesturl, ".bmp"))
	{
		$checkval = 1;
	}
	elseif (stristr($requesturl, ".png"))
	{
		$checkval = 1;
	}
	elseif (stristr($requesturl, ".gif"))
	{
		$checkval = 1;
	}


	// If not found anything unwanted yet, check to see if it's on the excludelist...
	if ($checkval == 0)
	{
		$excludelist = counterize_get_option('excludedip');

		$tmp_array = explode(",", $excludelist);
		$count = count($tmp_array);

		if ($excludelist != "" && $excludelist != " ")
		{
			for ($i=0; $i<$count; $i++)
			{
				if (strpos($remoteaddr, $tmp_array[$i]) === FALSE)
				{
					// Coming up...
				}
				else
				{
					// IP found on exclude-list - we don't want it!
					$checkval = 1;
				}
			}
		}
	}

	// DISABLED: This functionality is replaced with the admin functionality
	//			to enable/disable counting of certain users...
	//
	// let's check it is a logged in user.
	// If he's logged in, we don't count him.
	if ($checkval == 0)
	{
		$excluded_users = explode(",", counterize_get_option('excludedusers'));
		get_currentuserinfo();
		$tmp = count($excluded_users);
		if($user_ID != "" && $user_ID != " " && in_array($user_ID, $excluded_users))
		{
			$checkval = 1;
		}
	}

	// If checkval is still 0, then yes - we want to insert it...
	if ($checkval == 0)
	{
		// Replace %20's(spaces) in strings with a white-space
		// Man, someone should create a better checking-module... *sigh*
		$requesturl = str_replace("%20", " ", $requesturl);
		$referer = str_replace("%20", " ", $referer);
		$timestamp = gmdate("Y-m-d H:i:s", time() + ( get_option('gmt_offset') * 3600 )); // 3600 = 60 * 60

		$can_collect_ip = intval(counterize_get_option('enableip'));

		$agentID = counterize_getUserAgentID($useragent);
		$pageID = counterize_getPageID($requesturl);
		$refererID = counterize_getRefererID($referer);
		$keywordID = counterize_getKeywordID($referer);

		$sql = "INSERT INTO " . counterize_logTable() . " (IP, haship, timestamp, pageID, refererID, agentID) VALUES (";
		$sql .= "'" . ($can_collect_ip > 0 ? $wpdb->escape( $remoteaddr ) : 'unavailable') . "',";
		$sql .= "'" . substr( md5($wpdb->escape( $remoteaddr ) ), 1, 16) . "',";
		$sql .= "'" . $timestamp . "', '";
		$sql .= $pageID . "', '";
		$sql .= $refererID . "', '";
		$sql .= $agentID . "')";

		$results = $wpdb->query($sql);

		counterize_AddUserAgentVisit($agentID);
		counterize_AddPageVisit($pageID);
		counterize_AddRefererVisit($refererID);
		counterize_AddKeywordVisit($keywordID);
	}
}

// gives the useragentID back
function counterize_getUserAgentID($useragent)
{
	global $wpdb;
	$sql = "SELECT COUNT(agentID) "
		. " FROM " . counterize_agentsTable()
		. " WHERE name = '" . $wpdb->escape($useragent) . "'";

	if(!$wpdb->get_var($sql))
	{
		// create new agent
		list(
			$browser_name, $browser_code, $browser_ver, $browser_url,
			$os_name, $os_code, $os_ver, $browser_url,
			$pda_name, $pda_code, $pda_ver, $pda_url
		) = counterize_detect_browser($useragent);

		$sql = "INSERT INTO " . counterize_agentsTable() . " (name,count,browserName,browserCode,browserVersion,browserURL,osName,osCode,osVersion,osURL) VALUES (";
		$sql .= "'" . $wpdb->escape($useragent) . "',0,'" . $wpdb->escape($browser_name) . "','" . $wpdb->escape($browser_code) . "','" . $wpdb->escape($browser_ver) . "','" . $wpdb->escape($browser_url) . "','".$wpdb->escape($os_name) . "','" . $wpdb->escape($os_code) . "','" . $wpdb->escape($os_ver) . "','" . $wpdb->escape($os_url) . "')";
		$wpdb->query($sql);
	}

	$sql = "SELECT agentID "
		. " FROM ".counterize_agentsTable()
		. " WHERE name = '".$wpdb->escape($useragent)."'";
	return $wpdb->get_var($sql);
}

function counterize_AddUserAgentVisit($agentID)
{
	global $wpdb;
	if(is_numeric($agentID))
	{
		$sql = "UPDATE " . counterize_agentsTable()
			. " SET count = count + 1 "
			. " WHERE agentID = {$agentID}";
		$wpdb->query($sql);
	}
}

function counterize_AddKeywordVisit($keywordID)
{
	global $wpdb;
	if(is_numeric($keywordID)){
		$sql = "UPDATE " . counterize_keywordTable()
			. " SET count = count + 1 "
			. " WHERE keywordID = {$keywordID}";
		$wpdb->query($sql);
	}
}

// gives the pageID back
function counterize_getPageID($url)
{
	global $wpdb;
	global $post;
	$sql = "SELECT pageID "
		. " FROM " . counterize_pageTable()
		. " WHERE url = '" . $wpdb->escape($url) . "'";
	$pageID = $wpdb->get_var($sql);
	if(!$pageID)
	{
		// create new page
		$post_id = is_single() || $post->post_type == 'page'? $post->ID: 'NULL';
		$sql = "INSERT INTO " . counterize_pageTable() . " (url,count,postID) VALUES ('" . $wpdb->escape($url) . "',0," . $post_id . ")";
		$wpdb->query($sql);
		return $wpdb->insert_id;
	}
	return $pageID;
}

function counterize_AddPageVisit($pageID)
{
	global $wpdb;
	if(is_numeric($pageID))
	{
		$sql = "UPDATE " . counterize_pageTable()
			. " SET count = count + 1 "
			. " WHERE pageID = {$pageID}";
		$wpdb->query($sql);
	}
}

// gives the keywordID back
function counterize_getKeywordID($referer)
{
	global $wpdb;
	$ref = counterize_ref_analyzer($referer);
	$sql = "SELECT keywordID "
		. " FROM " . counterize_keywordTable()
		. " WHERE keyword = '" . $wpdb->escape($ref['keyword']) . "'";
	$keywordID = $wpdb->get_var($sql);

	if(!$keywordID)
	{
		// create new keyword
		$sql = "INSERT INTO " . counterize_keywordTable() . " (keyword,count) VALUES ('" . $wpdb->escape($ref['keyword']) . "',0)";
		$wpdb->query($sql);
		return $wpdb->insert_id;
	}
	return $keywordID;
}

// gives the refererID back
function counterize_getRefererID($referer)
{
	global $wpdb;
	$sql = "SELECT refererID "
		. " FROM " . counterize_refererTable()
		. " WHERE name = '" . $wpdb->escape($referer) . "'";
	$refererID = $wpdb->get_var($sql);
	if(!$refererID)
	{
		$keywordID = counterize_getKeywordID($referer);
		// create new referer
		$sql = "INSERT INTO ".counterize_refererTable()." (name,count,keywordID) VALUES (";
		$sql .= "'" . $wpdb->escape($referer) . "',0,{$keywordID})";
		$wpdb->query($sql);
		return $wpdb->insert_id;
	}
	return $refererID;
}

function counterize_AddRefererVisit($refererID)
{
	global $wpdb;
	if(is_numeric($refererID))
	{
		$sql = "UPDATE " . counterize_refererTable()
			. " SET count = count + 1 "
			. " WHERE refererID = {$refererID}";
		$wpdb->query($sql);
	}
}

function counterize_copyright()
{
	?>
	<br />
	<p style="text-align: center">
		<small><?php
			_e('Statistics recorded with <a href="http://www.gabsoftware.com/products/scripts/counterize/" title="Counterize - Statistics-plugin for Wordpress by GabSoftware">Counterize</a>', 'counterize');
			echo ' - ' . __('Version') . ' ' . counterize_get_version('major') . "." . counterize_get_version('minor') . "." . counterize_get_version('revision');
		?></small>
	</p>
	<p>
	<?php
}

function counterize_renderstats_vertical($rows, $header, $max_width = "500", $nofollow = true, $maxwidth = "100%", $shorten = true)
{
	global $counterize_plugin_url;
	$max_label = counterize_get_option('maxwidth');
	$items = 0;
	$complete_amount = 0;
	$max = 0;
	foreach($rows as $row)
	{
		$items++;
		$complete_amount += $row->amount;
		if($row->amount > $max)
		{
			$max = $row->amount;
		}
	}

	?>

			<table width="<?php echo $maxwidth; ?>">
				<tr class="alternate">
					<td style="width: 25%"><small><strong><?php _e($header,'counterize'); ?></strong></small></td>
					<td style="width: 10%"><small><strong><?php _e('Amount','counterize'); ?></strong></small></td>
					<td style="width: 65%"><small><strong><?php _e('Percentage','counterize'); ?></strong></small></td>
				</tr>

	<?php
	$counter = 0;
	foreach($rows as $row)
	{
		$percent = round($row->amount / $complete_amount * 100, 2);

		if( isset($row->amount) || array_key_exists('amount', get_object_vars($row)) )
		{
			$width = round($row->amount * $max_width / $max);
		}
		else
		{
			$width = 0;
		}

		$group = round($width / $max_width * 100);
		?>

				<tr<?php if($counter%2) { echo " class=\"alternate\""; } ?>>
					<td style="width: 25%" align="left">
						<small><?php
		if(strlen($row->label) > $max_label && $shorten == true)
		{
			$label = substr($row->label,0,$max_label) . '...';
		}
		else
		{
			$label = $row->label;
		}
		if( isset($row->url) || array_key_exists('url', get_object_vars($row)) )
		{
			echo "\n\t\t\t\t\t\t\t<a href=\"" . $row->url . "\"";
			if($nofollow)
			{
				echo " rel=\"nofollow\"";
			}
			echo " target=\"_blank\">\n\t\t\t\t\t\t\t\t" . $label . "\n\t\t\t\t\t\t\t</a>\n";
		}
		else
		{
			echo $label;
		}
		?>
						</small>
					</td>

					<td style="width: 10%">
						<small><?php echo $row->amount; ?></small>
					</td>

					<td style="width: 65%" align="left">
						<img	src="<?php echo $counterize_plugin_url;
				if ($group < 40)
						echo "/counterize_red.png";
				else if ($group < 80)
						echo "/counterize_yellow.png";
				else
						echo "/counterize_green.png";
				?>"
							style="height:8px; width:<?php echo $width ?>px; vertical-align:bottom"
							alt="<?php
				echo $header . ' - ' . htmlspecialchars($row->label2) . ' - ' . $row->amount . ' - ' . $percent . ' %';
				?>" />
						<small><strong><?php echo $percent; ?> %</strong></small>
					</td>
				</tr>

	<?php $counter++;
	}
	?>

			</table>

<?php
}


function counterize_renderstats_vertical_collapsed($rows_parents, $rows_all_child, $uniqueid, $header, $max_width = "500", $nofollow = true, $maxwidth = "100%", $shorten = true)
{
	global $counterize_plugin_url;
	$max_label = counterize_get_option('maxwidth');
	$items = 0;
	$complete_amount = 0;
	$max = 0;
	foreach($rows_parents as $parent)
	{
		$items++;
		$complete_amount += $parent->amount;
		if($parent->amount > $max)
		{
			$max = $parent->amount;
		}
	}

	?>

			<table width="<?php echo $maxwidth; ?>">
				<tr class="alternate">
					<td style="width: 25%"><small><strong><?php _e($header,'counterize'); ?></strong></small></td>
					<td style="width: 10%"><small><strong><?php _e('Amount','counterize'); ?></strong></small></td>
					<td style="width: 65%"><small><strong><?php _e('Percentage','counterize'); ?></strong></small></td>
				</tr>

	<?php
	$counter = 0;
	foreach($rows_parents as $parent)
	{
		$percent = round($parent->amount / $complete_amount * 100, 2);

		if( isset($parent->amount) || array_key_exists('amount', get_object_vars($parent)) )
		{
			$width = round($parent->amount * $max_width / $max);
		}
		else
		{
			$width = 0;
		}

		$group = round($width / $max_width * 100);
		?>

				<tr<?php if($counter % 2) { echo " class=\"alternate\""; } ?>>
					<td style="width: 25%" align="left">
						<?php
						$is_unknown = true;
						if($parent->code != 'unknown')
						{
							$is_unknown = false;
							?>

						<span><a id="counterize_parent_<?php echo $uniqueid . "_" . $counter; ?>" href="javascript:fold('counterize_parent_<?php echo $uniqueid . "_" . $counter; ?>', 'counterize_child_<?php echo $uniqueid . "_" . $counter; ?>');">[+]</a></span>

						<?php
						}
						?><small><?php
		if(strlen($parent->label) > $max_label && $shorten == true)
		{
			$label = substr($parent->label, 0, $max_label) . '...';
		}
		else
		{
			$label = $parent->label;
		}
		if( isset($parent->url) || array_key_exists('url', get_object_vars($parent)) )
		{
			echo "\n\t\t\t\t\t\t\t<a href=\"" . $parent->url . "\"";
			if($nofollow)
			{
				echo " rel=\"nofollow\"";
			}
			echo " target=\"_blank\">\n\t\t\t\t\t\t\t\t" . $label . "\n\t\t\t\t\t\t\t</a>\n";
		}
		else
		{
			echo $label;
		}
		?>
						</small>
					</td>

					<td style="width: 10%">
						<small><?php echo $parent->amount; ?></small>
					</td>

					<td style="width: 65%" align="left">
						<img	src="<?php echo $counterize_plugin_url;
				if ($group < 40)
						echo "/counterize_red.png";
				else if ($group < 80)
						echo "/counterize_yellow.png";
				else
						echo "/counterize_green.png";
				?>"
							style="height:8px; width:<?php echo $width; ?>px; vertical-align:bottom"
							alt="<?php
				echo $header . ' - ' . htmlspecialchars($parent->label2) . ' - ' . $parent->amount . ' - ' . $percent . ' %';
				?>" />
						<small><strong><?php echo $percent; ?> %</strong></small>
					</td>
				</tr>

				<?php
				if ( !$is_unknown)
				{
				?>

				<tr id="counterize_child_<?php echo $uniqueid . "_" . $counter; ?>" class="collapsed">
					<td colspan="3">
						<table style="background-color: #ddd;" width="<?php echo $maxwidth; ?>">

				<?php
					$rows_child = $rows_all_child[$counter];

					$child_items = 0;
					$child_complete_amount = 0;
					$child_max = 0;
					foreach($rows_child as $child)
					{
						$child_items++;
						$child_complete_amount += $child->amount;
						if($child->amount > $child_max)
						{
							$child_max = $child->amount;
						}
						if($child->label == "" || $child->label == " ")
						{
							$child->label = __('(no version)','counterize');
						}
					}

					$child_counter = 0;
					foreach($rows_child as $child)
					{


						$child_percent = round($child->amount / $child_complete_amount * 100, 2);

						if( $child_max > 0 && ( isset($child->amount) || array_key_exists('amount', get_object_vars($child)) ) )
						{
							$child_width = round($child->amount * $max_width / $child_max);
						}
						else
						{
							$child_width = 0;
						}

						$child_group = round($child_width / $max_width * 100);


						?>

								<tr<?php if($child_counter % 2) { echo " class=\"alternate\""; } ?>>

									<td style="width: 25%" align="left">
										<small><?php

						if(strlen($child->label) > $max_label && $shorten == true)
						{
							$child_label = substr($child->label, 0, $max_label) . '...';
						}
						else
						{
							$child_label = $child->label;
						}
						echo $child_label;
						?>

										</small>
									</td>

									<td style="width: 10%">
										<small><?php echo $child->amount; ?></small>
									</td>

									<td style="width: 65%" align="left">
										<img src="<?php echo $counterize_plugin_url;
						if ($child_group < 40)
							echo "/counterize_red.png";
						else if ($child_group < 80)
							echo "/counterize_yellow.png";
						else
							echo "/counterize_green.png";
						?>"
											style="height:8px; width:<?php echo $child_width; ?>px; vertical-align:bottom"
											alt="<?php
						echo $header . ' - ' . htmlspecialchars($parent->label . " " . $child->label) . ' - ' . $child->amount . ' - ' . $child_percent . ' %';
						?>" />
										<small><strong><?php echo $child_percent; ?> %</strong></small>
									</td>
								</tr>

						<?php

						$child_counter++;

					}

				?>

						</table>
					</td>

				</tr>
				<?php

				}

		$counter++;
	}
	?>

			</table>

	<?php
}


function counterize_renderstats($rows, $max_height = 80, $maxwidth = "100%")
{
	global $counterize_plugin_url;
	?>
	<table width="<?php echo $maxwidth; ?>">
		<tr>
	<?php

	$items = 0;
	$complete_amount = 0;
	$max = 0;

	foreach($rows as $row)
	{
		$items++;
		$complete_amount += $row->amount;
		if($row->amount > $max)
		{
			$max = $row->amount;
		}
	}

	$i = 0;
	foreach($rows as $row)
	{
		$percent = round($row->amount / $complete_amount * 100,2);

		if($row->amount)
		{
			$height = round($row->amount * $max_height / $max);
		}
		else
		{
			$height = 0;
		}

		$group = round($height / $max_height * 100);

		echo "<td style=\"width:3%\"";
		if($i%2)
		{
			print "class=\"alternate\"";
		}

		echo " align=\"center\" valign=\"bottom\"><small>";
		echo $row->amount;
		?>
		<br />
		<img src="<?php echo $counterize_plugin_url;
			if ($group < 40)
			{
				echo "/counterize_red.png";
			}
			else if ($group < 80)
			{
				echo "/counterize_yellow.png";
			}
			else
			{
				echo "/counterize_green.png";
			}
			?>" style="width:8px; height:<?php echo $height ?>px; vertical-align:bottom" alt="Statistics" />
			<?php
			echo "<br />{$percent}<br />%</small></td>";
			$i++;
	}
	?>
		</tr>
		<tr>
	<?php
	$i = 0;
	foreach($rows as $row)
	{
		echo "<td style=\"width:3%\"";
		if($i % 2)
		{
			echo "class=\"alternate\"";
		}
		echo " align=\"center\"><small><strong>{$row->label}</strong></small></td>";
		$i++;
	}
	?>
		</tr>
	</table>
	<?php
}






function counterize_show_history()
{
	$howmany = __("Latest entries", 'counterize');

	$amount = counterize_get_option('amount');
	if ($amount == "" || $amount == " ")
	{
		$amount = 50;
	}

	$howmany = "{$howmany} ({$amount})";

	$entries = counterize_getentries($amount);

	?>

	<div class="wrap">
	<h2><?php echo $howmany; ?></h2>

	<a href="index.php?page=counterize_dashboard"><?php _e("Reset Filters", 'counterize'); ?></a></br>
	</br>
	<?php
		$killmass_url    = "index.php?page=counterize_dashboard&amp;killmass=yes";
		$killmass_link = wp_nonce_url( $killmass_url, "action_killmass" );
	?>
	<form method="post"
		action="<?php echo $killmass_link; ?>"
		name="tablesForm" id="tablesForm">
	<table width="100%" cellpadding="3" cellspacing="3">
		<tr class="alternate">
			<td scope="col" width="2%"><strong><?php _e("Select", 'counterize'); ?></strong></td>
			<td scope="col" width="6%"><strong><?php _e("ID", 'counterize'); ?></strong></td>
			<td scope="col" width="13%"><strong><?php _e("IP",'counterize'); ?></strong></td>
			<td scope="col" style="width: 14%"><strong><?php _e("Timestamps", 'counterize'); ?></strong></td>
			<td scope="col" style="width: 25%"><strong><?php _e("URL", 'counterize'); ?></strong></td>
			<td scope="col" style="width: 25%"><strong><?php _e("Referers", 'counterize'); ?></strong></td>
			<td scope="col" style="width: 14%"><strong><?php _e("User-agents", 'counterize'); ?></strong></td>
			<td scope="col" style="width: 25%"><strong><?php _e("Keywords", 'counterize'); ?></strong></td>
			<td scope="col" style="width: 3%"><strong><?php _e("Kill", 'counterize'); ?></strong></td>
		</tr>

	<?php


		$url    = "index.php?page=counterize_dashboard";

		$uri_filter_link     = wp_nonce_url( $url, "action_uri_filter" );
		$ip_filter_link      = wp_nonce_url( $url, "action_ip_filter" );
		$referer_filter_link = wp_nonce_url( $url, "action_referer_filter" );
		$agent_filter_link   = wp_nonce_url( $url, "action_agent_filter" );
		$uri_kill_link       = wp_nonce_url( $url, "action_uri_kill" );


	$i=0;
	$entrycounter = 1;
	foreach($entries as $entry)
	{
		?>
		<tr <?php if($i%2) { print "class=\"alternate\""; } ?>>
			<td scope="col">
				<?php echo ($entrycounter . "  "); $entrycounter++; ?>
				<small>
					<input type="checkbox"
					name='counterize_killemall[<?php echo $entry->id; ?>]'
					value="<?php echo $entry->id; ?>" />
				</small>
			</td>
			<td>
				<small>
					<?php echo $entry->id; ?>
				</small>
			</td>


			<td scope="col">
				<small>
					<?php printf('<a href="%s%s" target="_blank">%s</a>', counterize_get_option('whois'), $entry->IP, $entry->IP); ?>
					(<a href="<?php echo $ip_filter_link . "&amp;ipfilter={$entry->IP}"; ?>">F</a>)
					(<a target="_blank" href="http://whatismyipaddress.com/ip/<?php echo $entry->IP; ?>">V</a>)
				</small>
			</td>

			<td scope="col">
				<small><?php echo $entry->timestamp; ?> </small>
			</td>
			<td scope="col">
				<small>
					<?php echo "<a href=\"" . $entry->url . "\" target=\"_blank\">" . wordwrap($entry->url, 30, "\n", 1); ?>
					</a> (<a href="<?php echo $uri_filter_link . "&amp;urifilter={$entry->url}"; ?>">F</a>)
				</small>
			</td>
			<td scope="col">
				<small>
				<?php
				if ($entry->referer != "unknown")
				{
					echo "<a href=\"" . $entry->referer . "\" target=\"_blank\">" . wordwrap($entry->referer, 30, "\n", 1) . "</a>";
					?> (<a
					href="<?php echo $referer_filter_link . "&amp;refererfilter={$entry->referer}"; ?>">F</a>)
					<?php
				}
				else
				{
					echo wordwrap($entry->referer, 30, "\n", 1);
				}
				?>
				</small>
			</td>
			<td scope="col">
				<small>
					<?php echo counterize_browser_string($entry->useragent , true, '<br>'); ?> (<a href="<?php echo $agent_filter_link . "&amp;agentfilter={$entry->useragent}"; ?>">F</a>)
				</small>
			</td>
			<td>
				<small><?php echo $entry->keyword; ?></small>
			</td>
			<td scope="col">
				<a href="javascript:conf('<?php echo $uri_kill_link . "&amp;kill={$entry->id}"; ?>');"><font color="red" size="+1">X</font></a>
			</td>
			<?php
			$i++;
	}
	?>
		</tr>
	</table>

	<input type="button" name="CheckAll"
		value="<?php _e("Check All",'counterize'); ?>"
		onClick="checkAll(document.tablesForm)" /> <input type="button"
		name="UnCheckAll" value="<?php _e("Uncheck All", 'counterize'); ?>"
		onClick="uncheckAll(document.tablesForm)" /> <input type="SUBMIT"
		value="<?php _e("Delete selected Entries", 'counterize'); ?>" /></form>
	</div>

<?php

}

function counterize_updateText($text="Configuration updated", $color="red")
{
	echo "<div id=\"message\" class=\"updated fade\"><p><font color=\"{$color}\">";
	_e($text, 'counterize');
	echo "</font></p></div>";
}

//display all the statistics
function counterize_showStats($admin = false)
{
	if(!counterize_getamount())
	{
		_e("There's no data in the database - You can't see stats until you have data.", 'counterize');
		return;
	}
	?>
	</p>
	<div class="wrap">
	<h2><?php _e('Hit Counter', "counterize");?></h2>

	<?php counterize_get_hits(); ?>

	</div>

		<?php
		// Amount to pass as option to the graphs...
		$amount2 = counterize_get_option('amount2');
		if ($amount2 == "" || $amount2 == " " || !is_numeric($amount2))
		{
			$amount2 = 10;
		}
		$width = 250;
		?>

	<div class="wrap">
	<h2><?php _e('Visits based on day of month', 'counterize');?></h2>
		<?php
		if($admin)
		{
			counterize_renderstats(counterize_getdailystats());
		}
		else
		{
			counterize_renderstats_vertical(counterize_getdailystats(), __("Day", 'counterize'), $width);
		}
		?></div>

	<div class="wrap">
	<h2><?php _e('Visits based on day of month (only this month)', 'counterize');?></h2>
		<?php
		if($admin)
		{
			counterize_renderstats(counterize_getdailystats(true));
		}
		else
		{
			counterize_renderstats_vertical(counterize_getdailystats(true), __("Day", 'counterize'), $width);
		}
		?></div>


	<div class="wrap">
	<h2><?php _e('Visits based on day of week', 'counterize');?></h2>
		<?php
		if($admin)
		{
			counterize_renderstats(counterize_getweeklystats());
		}
		else
		{
			counterize_renderstats_vertical(counterize_getweeklystats(), __("Day", 'counterize'), $width);
		}
		?></div>

	<div class="wrap">
	<h2><?php _e('Visits based on month', 'counterize');?></h2>
		<?php
		if($admin)
		{
			counterize_renderstats(counterize_getmonthlystats());
		}
		else
		{
			counterize_renderstats_vertical(counterize_getmonthlystats(), __("Month", 'counterize'), $width);
		}
		?></div>

	<div class="wrap">
	<h2><?php _e('Visits based on hour of day', 'counterize');?></h2>
		<?php
		if($admin)
		{
			counterize_renderstats(counterize_gethourlystats());
		}
		else
		{
			counterize_renderstats_vertical(counterize_gethourlystats(), __("Hour", 'counterize'), $width);
		}
		?></div>

	<!-- Pages -->
	<div class="wrap">
		<h2><?php echo __("Most visited pages", 'counterize') . " (" . $amount2 .")"; ?></h2>
		<?php counterize_most_visited_pages($amount2, $width); ?>
	</div>

		<div class="wrap">
		<h2><?php echo __("Most visited pages (last 24 hours)", 'counterize') . "  (" . $amount2 .")"; ?></h2>
		<?php counterize_most_visited_pages24hrs($amount2, $width); ?>
	</div>

	<?php
	if($admin)
	{
		?>

		<div class="wrap">
			<h2><?php echo __("Most active IPs ", 'counterize') . "(" . $amount2 . ")"; ?></h2>
			<?php counterize_most_visited_IPs($amount2, $width); ?>
		</div>

		<div class="wrap">
			<h2><?php echo __("Most active IPs (last 24 hours) ", 'counterize') . "(" . $amount2 . ")"; ?></h2>
			<?php counterize_most_visited_IPs24hrs($amount2, $width); ?>
		</div>

		<?php
	}
	?>

	<!-- Referers -->
	<div class="wrap">
		<h2><?php echo __("Most seen referers", 'counterize') . " (" . $amount2 . ")"; ?></h2>
		<?php counterize_most_visited_referrers($amount2, $width); ?>
	</div>

	<div class="wrap">
		<h2><?php echo __("Most seen referers (last 24 hours)", 'counterize') . " (" . $amount2 . ")"; ?></h2>
		<?php counterize_most_visited_referrers24hrs($amount2, $width); ?>
	</div>

	<!-- Browsers -->
	<div class="wrap">
		<h2><?php echo __("Most used browsers (click [+] for details)",'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_used_browsers_collapsible($amount2, $amount2, $width); ?>
	</div>

	<div class="wrap">
		<h2><?php echo __("Most used browsers (without version)", 'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_used_browsers_without_version($amount2, $width); ?>
	</div>

	<div class="wrap">
		<h2><?php echo __("Most used browsers", 'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_used_browsers($amount2, $width); ?>
	</div>

	<!-- OS -->
	<div class="wrap">
		<h2><?php echo __("Most used operating systems (click [+] for details)",'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_used_os_collapsible($amount2, $amount2, $width); ?>
	</div>

	<div class="wrap">
		<h2><?php echo __("Most used operating systems (without version)",'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_used_os_without_version($amount2, $width); ?>
	</div>

	<div class="wrap">
		<h2><?php echo __("Most used operating systems",'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_used_os($amount2, $width); ?>
	</div>

	<!-- Keywords -->
	<div class="wrap">
		<h2><?php echo __("Most searched keywords",'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_searched_keywords($amount2, $width); ?>
	</div>

	<div class="wrap">
		<h2><?php echo __("Most searched keywords today",'counterize') . " (" . $amount2 . ")";?></h2>
		<?php counterize_most_searched_keywords_today($amount2, $width); ?>
	</div>

	<?php
}

//Call the proper function depending of the $pattern
function counterize_check_data($data, $pattern, $function, $argc=0)
{
	if ( is_callable( $function ) && is_numeric( $argc ) )
	{
		while(preg_match($pattern, $data, $matches))
		{
			ob_start();

			$correct = true;
			if( $argc == 1 && isset( $matches[2] ) )
			{
				if( is_numeric( $matches[2] ) )
				{
					$ret = call_user_func($function, $matches[2]);
				}
				else
				{
					$correct = false;
				}
			}
			elseif( $argc == 2 && isset( $matches[2] ) && isset( $matches[3] ) )
			{
				if( is_numeric( $matches[2] ) && is_numeric( $matches[3] ) )
				{
					$ret = call_user_func($function, $matches[2], $matches[3]);
				}
				else
				{
					$correct = false;
				}
			}
			else
			{
				$ret = call_user_func($function);
			}

			if( $correct )
			{
				$content = ob_get_contents();
				ob_end_clean();
				$replace_pattern = $pattern;
				$data = preg_replace($replace_pattern, $content, $data);
			}
		}
	}
	return $data;
}


//Allow the authors to integrate charts into their posts using the following syntax (in HTML mode):
// <!-- keyword -->
function counterize_filter_callback($data)
{
	//search if the post has the magic word
	$pattern = '/(\<\!|\#)\-\-\s*counterize_stats/';

	if (preg_match($pattern, $data) )
	{
		//we probably want to add some statistics. Lets verify:

		//<!-- counterize_stats --> : Shows all the stats
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats\s*\-\-(\>|\#)/'                              , 'counterize_showStats'                         , 0);

		//<!-- counterize_stats_hits --> : Shows a table containing statistics about hits
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_hits\s*\-\-(\>|\#)/'                         , 'counterize_get_hits'                          , 0);

		//<!-- counterize_stats_browsers_nb --> : Shows a list of the nb most used browsers (with their version)
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_browsers_([0-9]+)\s*\-\-(\>|\#)/'            , 'counterize_most_used_browsers'                , 1);

		//<!-- counterize_stats_browsers_nover_nb --> : Shows a list of the nb most used browsers (without their version)
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_browsers_nover_([0-9]+)\s*\-\-(\>|\#)/'      , 'counterize_most_used_browsers_without_version', 1);

		//<!-- counterize_stats_browsers_mixed_nb1_nb2 --> : Shows a list of the nb1 most used browsers (clicking each entry will show the version details)
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_browsers_mixed_([0-9]+)_([0-9]+)\s*\-\-(\>|\#)/' , 'counterize_most_used_browsers_collapsible', 2);

		//<!-- counterize_stats_os_nb --> : Shows a list of the nb most used operating systems (with their version)
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_os_([0-9]+)\s*\-\-(\>|\#)/'                  , 'counterize_most_used_os'                      , 1);

		//<!-- counterize_stats_os_nover_nb --> : Shows a list of the nb most used operating systems (without their version)
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_os_nover_([0-9]+)\s*\-\-(\>|\#)/'            , 'counterize_most_used_os_without_version'      , 1);

		//<!-- counterize_stats_os_mixed_nb1_nb2 --> : Shows a list of the nb1 most used operating systems (clicking each entry will show the version details)
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_os_mixed_([0-9]+)_([0-9]+)\s*\-\-(\>|\#)/'   , 'counterize_most_used_os_collapsible'          , 2);

		//<!-- counterize_stats_ip_nb --> : Shows a list of the nb most active ips
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_ip_([0-9]+)\s*\-\-(\>|\#)/'                  , 'counterize_most_visited_IPs'                  , 1);

		//<!-- counterize_stats_ip_24hrs_nb --> : Shows a list of the nb most active ips during the last 24 hours
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_ip_24hrs_([0-9]+)\s*\-\-(\>|\#)/'                  , 'counterize_most_visited_IPs24hrs'             , 1);

		//<!-- counterize_stats_pages_nb --> : Shows a list of the nb most visited pages
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_pages_([0-9]+)\s*\-\-(\>|\#)/'               , 'counterize_most_visited_pages'                , 1);

		//<!-- counterize_stats_pages_24hrs_nb --> : Shows a list of the nb most visited pages during the last 24 hours
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_pages_24hrs_([0-9]+)\s*\-\-(\>|\#)/'         , 'counterize_most_visited_pages24hrs'           , 1);

		//<!-- counterize_stats_referers_nb --> : Shows a list of the nb most seen referers
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_referers_([0-9]+)\s*\-\-(\>|\#)/'            , 'counterize_most_visited_referrers'            , 1);

		//<!-- counterize_stats_referers_24hrs_nb --> : Shows a list of the nb most seen referers during the last 24 hours
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_referers_24hrs_([0-9]+)\s*\-\-(\>|\#)/'      , 'counterize_most_visited_referrers24hrs'       , 1);

		//<!-- counterize_stats_keywords_nb --> : Shows a list of the nb most searched keywords
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_keywords_([0-9]+)\s*\-\-(\>|\#)/'            , 'counterize_most_searched_keywords'            , 1);

		//<!-- counterize_stats_keywords_today_nb --> : Shows a list of the nb most searched keywords today
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_keywords_today_([0-9]+)\s*\-\-(\>|\#)/'      , 'counterize_most_searched_keywords_today'      , 1);

		//<!-- counterize_stats_copyright --> : Shows a small copyright notice
		$data = counterize_check_data($data, '/(\<\!|\#)\-\-\s*counterize_stats_copyright\s*\-\-(\>|\#)/'                    , 'counterize_copyright'                         , 0);

	}

	return $data;
}

function counterize_pagefooter()
{
	?>
	<div class="wrap">
		<?php _e('<strong>Need help? </strong>Go to <a href="http://www.gabsoftware.com/products/scripts/counterize/">the Counterize homepage</a> and search inside the comments. Someone may have had the same question or problem as you...', 'counterize'); ?>
	</div>
	<?php
}








//Executed when Wordpress is loaded but before any header is sent.
//Inside we register our javascript and stylesheet files.
function counterize_init_callback()
{
	global $counterize_plugin_url;
	global $counterize_plugin_dir;

	if(function_exists('load_plugin_textdomain'))
	{
		load_plugin_textdomain( 'counterize', $counterize_plugin_dir, plugin_basename(dirname(__FILE__)));
	}

	wp_register_script( 'counterize_javascript', $counterize_plugin_url . '/counterize.js.php', __FILE__);
	wp_register_style( 'counterize_stylesheet', $counterize_plugin_url . '/counterize.css.php', __FILE__);

	add_action('wp_print_scripts', 'counterize_javascript_head_callback');
	add_action('wp_print_styles', 'counterize_stylesheet_head_callback');




	//Do the administrative stuff only if we are in the administration area
	if( is_admin() )
	{
		//require_once(ABSPATH . 'wp-includes/pluggable.php');
		if( current_user_can( 'manage_options' ) )
		{
			//include admin-related files
			require_once("counterize_admin.php");
			require_once("counterize_dashboard.php");

			//check if we need to install Counterize
			counterize_should_install();

			//Add admin actions
			add_action( 'admin_init', 'counterize_admin_init_callback' );
			add_action( 'admin_menu', 'counterize_add_pages_admin_callback' );
		}
		else
		{
			//Non admin pages
			$capability = counterize_get_option('display_dashboard_capability');
			if ($capability === FALSE)
			{
				$capability = 'manage_options';
				counterize_set_option('display_dashboard_capability', $capability);
			}

			if( current_user_can( $capability ) )
			{
				require_once("counterize_dashboard.php");
				add_action( 'admin_menu', 'counterize_add_pages_callback' );
			}
		}
	}
}


//Add the Counterize widget into the dashboard
function add_counterize_dashboard_widget_callback()
{
	wp_add_dashboard_widget( 'dashboard_counterize', __('Counterize Status', 'counterize'), 'counterize_dashboard_callback');
}

//Add the Counterize javascript file to the <head> section
function counterize_javascript_head_callback() {
	wp_enqueue_script('counterize_javascript');
}

//Add the Counterize stylesheet file to the <head> section
function counterize_stylesheet_head_callback() {
	wp_enqueue_style('counterize_stylesheet');
}

//add a Configure link in the plugins page
function counterize_filter_plugin_actions_callback($links, $file)
{
	$settings_link = '<a href="options-general.php?page=counterize_options_page_id">' . __('Configure', 'counterize') . '</a>';
	array_unshift( $links, $settings_link ); // add the configure link before other links
	return $links;
}

//Add options menus an pages for administrators
function counterize_add_pages_admin_callback()
{
	//specify that we want to add a dashboard widget
	add_action('wp_dashboard_setup', 'add_counterize_dashboard_widget_callback' );

	//add the Counterize dashboard to the Dashboard section
	//$counterize_dashboard_page_handle = add_submenu_page('index.php',__('Counterize', 'counterize'), __('Counterize', 'counterize'), 'activate_plugins', "counterize_dashboard", 'counterize_display_dashboard_page_callback');
	$counterize_dashboard_page_handle = add_dashboard_page(__('Counterize', 'counterize'), __('Counterize', 'counterize'), 'edit_dashboard', "counterize_dashboard", 'counterize_display_dashboard_page_callback');
	//$counterize_dashboard_page_handle = add_dashboard_page(__('Counterize', 'counterize'), __('Counterize', 'counterize'), 'read', "counterize_dashboard", 'counterize_display_dashboard_page_callback');


	//add the Counterize options page
	$counterize_options_page_handle = add_options_page(__('Counterize Options', 'counterize'), __('Counterize', 'counterize'), 'manage_options', "counterize_options_page_id", 'counterize_options_page_callback');

	//specify that we want to add the javascript code only for the Counterize dashboard and options pages
	add_action( "admin_print_scripts-{$counterize_dashboard_page_handle}", 'counterize_javascript_head_callback' );
	add_action( "admin_print_styles-{$counterize_dashboard_page_handle}", 'counterize_stylesheet_head_callback' );
	add_action( "admin_print_scripts-{$counterize_options_page_handle}", 'counterize_javascript_head_callback' );
	add_action( "admin_print_styles-{$counterize_options_page_handle}", 'counterize_stylesheet_head_callback' );

	//specify that we want to alter the links in the plugins page
	add_filter( 'plugin_action_links_' . plugin_basename(__FILE__), 'counterize_filter_plugin_actions_callback', 10, 2 );
}

function counterize_add_pages_callback()
{
	//specify that we want to add a dashboard widget
	add_action('wp_dashboard_setup', 'add_counterize_dashboard_widget_callback' );
	$counterize_dashboard_page_handle = add_dashboard_page(__('Counterize', 'counterize'), __('Counterize', 'counterize'), 'read', "counterize_dashboard", 'counterize_display_dashboard_page_callback');

	//specify that we want to add the javascript code only for the Counterize dashboard
	add_action( "admin_print_scripts-{$counterize_dashboard_page_handle}", 'counterize_javascript_head_callback' );
	add_action( "admin_print_styles-{$counterize_dashboard_page_handle}", 'counterize_stylesheet_head_callback' );
}


//check is Counterize should be installed or upgraded
function counterize_should_install()
{
	$MajorVersion = get_option('counterize_MajorVersion');
	if ($MajorVersion === FALSE)
	{
		$MajorVersion = counterize_get_version('major');
		$MinorVersion = counterize_get_version('minor');
		$Revision     = counterize_get_version('revision');
	}
	else
	{
		$MajorVersion = get_option('counterize_MajorVersion', 1);
		$MinorVersion = get_option('counterize_MinorVersion', 0);
		$Revision     = get_option('counterize_Revision'    , 0);
	}

	global $counterize_MajorVersion;
	global $counterize_MinorVersion;
	global $counterize_Revision;

	if ($MajorVersion != $counterize_MajorVersion || $MinorVersion != $counterize_MinorVersion || $Revision != $counterize_Revision)
	{
		require_once("counterize_install.php");
		counterize_install($counterize_MajorVersion, $counterize_MinorVersion, $counterize_Revision);
	}
}

add_action( 'init', 'counterize_init_callback' );

// Create API hook instead of placing code in the header.php-file
add_action('wp_head', 'counterize_add_callback', 1);
add_filter('the_content', 'counterize_filter_callback');

?>
