<?php

// security check
if ( !defined( 'WP_PLUGIN_DIR') || !current_user_can( 'manage_options' ) )
{
	die("There is nothing to see here...");
}

$counterize_plugin_dir = WP_PLUGIN_DIR . '/' . plugin_basename(dirname(__FILE__));
$counterize_plugin_url = WP_PLUGIN_URL . '/' . plugin_basename(dirname(__FILE__));

//Content of the Counterize options page
function counterize_options_page_callback()
{
	global $wpdb;

	//Not admin? Go away.
	if ( ! current_user_can( 'manage_options' ) )
	{
		die ( __( "You don't have sufficient privileges to display this page", 'counterize' ) );
	}

	$MajorVersion	= counterize_get_version('major');
	$MinorVersion	= counterize_get_version('minor');
	$Revision		= counterize_get_version('revision');

	?>

	<div class="wrap">
		<div class="icon32" id="icon-options-general"></div>

		<h2><?php echo __('Counterize configuration - Version ', "counterize") . $MajorVersion . '.' . $MinorVersion . '.' . $Revision; ?></h2>

		<form method="post" action="options.php">

			<?php
			settings_fields( 'counterize_options_group' );
			do_settings_sections( 'counterize_options_page_id' );
			?>

			<p class="submit">
				<input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
			</p>
		</form>

	</div>

	<?php

	counterize_pagefooter();
}

//Executed when Wordpress initialize the administration section.
//Inside we register our javascript ans stylesheet files.
function counterize_admin_init_callback()
{
	global $counterize_plugin_url;
	wp_register_script( 'counterize_javascript', $counterize_plugin_url . '/counterize.js.php', __FILE__);
	wp_register_style( 'counterize_stylesheet', $counterize_plugin_url . '/counterize.css.php', __FILE__);


	//register option group
	register_setting( 'counterize_options_group', 'counterize_options', 'counterize_options_validate_callback' );

	//add sections
	add_settings_section('counterize_options_section_appearance', __("Appearance"         , "counterize"), 'counterize_options_display_section_appearance_callback', 'counterize_options_page_id');
	add_settings_section('counterize_options_section_exclusions', __("Exclusions"         , "counterize"), 'counterize_options_display_section_exclusions_callback', 'counterize_options_page_id');
	add_settings_section('counterize_options_section_ip'        , __("IP-related"         , "counterize"), 'counterize_options_display_section_ip_callback'        , 'counterize_options_page_id');
	add_settings_section('counterize_options_section_db'        , __("Database operations", "counterize"), 'counterize_options_display_section_db_callback'        , 'counterize_options_page_id');


	//section Appearance
	add_settings_field('counterize_setting_amount'  , __("Rows to show in latest entries", "counterize") , 'counterize_options_display_amount_callback'  , 'counterize_options_page_id', 'counterize_options_section_appearance');
	add_settings_field('counterize_setting_amount2' , __("Rows to show in vertical charts", "counterize"), 'counterize_options_display_amount2_callback' , 'counterize_options_page_id', 'counterize_options_section_appearance');
	add_settings_field('counterize_setting_maxwidth', __("Maximum width for labels", "counterize")       , 'counterize_options_display_maxwidth_callback', 'counterize_options_page_id', 'counterize_options_section_appearance');
	add_settings_field('counterize_setting_display_dashboard_capability', __("Capability needed to display the Counterize dashboard", "counterize")       , 'counterize_options_display_display_dashboard_capability_callback', 'counterize_options_page_id', 'counterize_options_section_appearance');

	//section Exclusions
	add_settings_field('counterize_setting_excludedusers', __("Select users that may not be counted (press Ctrl for multi-selection)", "counterize")                   , 'counterize_options_display_excludedusers_callback', 'counterize_options_page_id', 'counterize_options_section_exclusions');
	add_settings_field('counterize_setting_excludedip'   , __("Comma-separated list of IP to exclude from the statistics", "counterize")                               , 'counterize_options_display_excludedip_callback'   , 'counterize_options_page_id', 'counterize_options_section_exclusions');
	add_settings_field('counterize_setting_logbots'      , __("Enable/disable logging of <a href=\"http://en.wikipedia.org/wiki/Internet_bot\">bots</a>", "counterize"), 'counterize_options_display_logbots_callback'      , 'counterize_options_page_id', 'counterize_options_section_exclusions');


	//section IP
	add_settings_field('counterize_setting_enableip', __("My country allows to collect IPs", "counterize"), 'counterize_options_display_enableip_callback', 'counterize_options_page_id', 'counterize_options_section_ip');
	add_settings_field('counterize_setting_whois'   , __("Whois URL string"                , "counterize"), 'counterize_options_display_whois_callback'   , 'counterize_options_page_id', 'counterize_options_section_ip');

	//section DB
	add_settings_field('counterize_setting_flushdb'   , __("Flush the Counterize Database", "counterize"), 'counterize_options_display_flushdb_callback'   , 'counterize_options_page_id', 'counterize_options_section_db');
	add_settings_field('counterize_setting_refreshua' , __("Refresh the user-agent table" , "counterize"), 'counterize_options_display_refreshua_callback' , 'counterize_options_page_id', 'counterize_options_section_db');
	add_settings_field('counterize_setting_deletebots', __("Manually delete the bots"     , "counterize"), 'counterize_options_display_deletebots_callback', 'counterize_options_page_id', 'counterize_options_section_db');
}

//display the Appearance section
function counterize_options_display_section_appearance_callback()
{
	echo '<p>' . __('This section concerns the look and feel of Counterize.', 'counterize') . '</p>';
}

//display the Exclusions section
function counterize_options_display_section_exclusions_callback()
{
	echo '<p>' . __('This section allows you to set some exclusions.', 'counterize') . '</p>';
}

//display the IP-related section
function counterize_options_display_section_ip_callback()
{
	echo '<p>' . __('This section is about IP-related options.', 'counterize') . '</p>';
}

//display the Database operations section
function counterize_options_display_section_db_callback()
{
	echo '<p>' . __('Database maintenance operations are in this section.', 'counterize') . '</p>';
}

function counterize_options_display_amount_callback()
{
	$options = get_option('counterize_options' );

	//set a default value if no value is set
	if ( ! isset( $options['amount']) || empty( $options['amount']) )
	{
		$options['amount'] = 50;
	}
	echo "<input id='counterize_setting_amount' name='counterize_options[amount]' size='10' type='text' value='{$options['amount']}' />";
	echo "<br /><small><strong>0</strong>" . __(' to view all (not recommended because this exposes you to get an "out of memory" error)', "counterize") . "</small>";
}
function counterize_options_display_amount2_callback()
{
	$options = get_option('counterize_options');
	//set a default value if no value is set
	if ( ! isset( $options['amount2']) || empty( $options['amount2']) )
	{
		$options['amount2'] = 50;
	}
	echo "<input id='counterize_setting_amount2' name='counterize_options[amount2]' size='10' type='text' value='{$options['amount2']}' />";
}
function counterize_options_display_maxwidth_callback()
{
	$options = get_option('counterize_options');
	//set a default value if no value is set
	if ( ! isset( $options['maxwidth']) || empty( $options['maxwidth']) )
	{
		$options['maxwidth'] = 50;
	}
	echo "<input id='counterize_setting_maxwidth' name='counterize_options[maxwidth]' size='10' type='text' value='{$options['maxwidth']}' />";
}

function counterize_options_display_display_dashboard_capability_callback()
{
	global $wp_roles;
	$options = get_option('counterize_options');
	//set a default value if no value is set
	if ( ! isset( $options['display_dashboard_capability']) || empty( $options['display_dashboard_capability']) )
	{
		$options['display_dashboard_capability'] = 'manage_options';
	}
	echo "
	<select id='counterize_setting_display_dashboard_capability' name='counterize_options[display_dashboard_capability]' />
	";

	//retrieve all the capabilities of the current user (usually an administrator)
	$current_user = wp_get_current_user();
	$current_user->get_role_caps();
	$capabilities = $current_user->allcaps;
	ksort($capabilities);
	foreach ($capabilities as $cap=>$whatever)
	{
		echo "
		<option value='{$cap}'" . ( $options['display_dashboard_capability'] == $cap ? "selected='selected'" : "" ) . ">{$cap}</option>
		";
	}

	echo "
	</select>
	";
}

function counterize_options_display_excludedusers_callback()
{
	global $wpdb;
	$options = get_option('counterize_options');
	$excluded_users	= explode( ",", counterize_get_option('excludedusers') );

	$sql = "SELECT * FROM {$wpdb->users}";
	$users = $wpdb->get_results($sql);

	?>

	<select id='counterize_setting_excludedusers' name='counterize_options[excludedusers][]' multiple='multiple' size='10' style='height: auto;'>

	<?php
	foreach ($users as $user)
	{
		?>

		<option value="<?php echo $user->ID; ?>" <?php if(in_array($user->ID, $excluded_users)){ echo "selected='selected'"; } ?>><?php echo $user->display_name . "&nbsp;&nbsp;&nbsp;[" . $user->user_login . "]"; ?>

		<?php
	}
	?>

	</select>

	<?php
}

function counterize_options_display_excludedip_callback()
{
	$options = get_option('counterize_options');
	echo "<textarea id='counterize_setting_excludedip' name='counterize_options[excludedip]' cols='50' rows='3'>{$options['excludedip']}</textarea>";
}

function counterize_options_display_logbots_callback()
{
	$options = get_option('counterize_options');
	echo "<input id='counterize_setting_logbots' name='counterize_options[logbots]' type='checkbox' value='1'" . ($options['logbots'] > 0 ? " checked='checked'" : " ") . " />";
}

function counterize_options_display_enableip_callback()
{
	$options = get_option('counterize_options');
	echo "<input id='counterize_setting_enableip' name='counterize_options[enableip]' type='checkbox' value='1'" . ($options['enableip'] > 0 ? " checked='checked'" : " ") . " />";
}

function counterize_options_display_whois_callback()
{
	$options = get_option('counterize_options');
	echo "<input id='counterize_setting_whois' name='counterize_options[whois]' size='60' type='text' value='{$options['whois']}' />";
	echo "<br /><small>" . __('Example:', "counterize") . " http://www.ripe.net/whois?form_type=simple&searchtext=</small>";
}

/* these options don't have any value stored in the database */
function counterize_options_display_flushdb_callback()
{
	$options = get_option('counterize_options');
	echo "<input id='counterize_setting_flushdb' name='counterize_options[flushdb]' type='checkbox' value='wantflushdb' />";
	echo "<br /><small><strong>" . __("(This means deleting all records and all stats! Don't forget to backup your database before if your data is important.)", "counterize") . "</strong></small>";

	if( isset( $_SESSION["counterize_flusheddb"] ) )
	{
		if( $_SESSION["counterize_flusheddb"]== "flushed")
		{
			counterize_updateText(__("Database flushed!", "counterize"));
			unset( $_SESSION["counterize_flusheddb"] );
		}
	}
}

function counterize_options_display_refreshua_callback()
{
	$options = get_option('counterize_options');
	echo "<input id='counterize_setting_refreshua' name='counterize_options[refreshua]' type='checkbox' value='wantrefreshua' />";
	echo "<br /><small>" . __('(You should do this if you just upgraded from a previous version!)', "counterize") . "</small>";

	if ( isset( $_SESSION["counterize_refreshedua"] ) )
	{
		if( $_SESSION["counterize_refreshedua"] == "refreshed" )
		{
			counterize_updateText(__("User-agent table refreshed!", "counterize"));
			unset( $_SESSION["counterize_refreshedua"] );
		}
	}
}

function counterize_options_display_deletebots_callback()
{
	$options = get_option('counterize_options');
	echo "<input id='counterize_setting_deletebots' name='counterize_options[deletebots]' type='checkbox' value='wantdeletebots' />";
	echo "<br /><small>" . __('(You should do this if you modified botlist.txt)', "counterize") . "</small>";

	if( isset( $_SESSION["counterize_deletedbots"] ) )
	{
		if($_SESSION["counterize_deletedbots"] == "deleted")
		{
			counterize_updateText(__("Bots have been deleted!", "counterize"));
			unset( $_SESSION["counterize_deletedbots"] );
		}
	}
}

//validate the input data
function counterize_options_validate_callback($input)
{
	//load the current options
	$newinput = get_option('counterize_options');

	if( isset( $input['amount'] ) )
	{
		$newinput['amount'] = trim($input['amount']);
		if(!preg_match('/^[0-9]+$/i', $newinput['amount'])) {
			$newinput['amount'] = '50';
		}
	}

	if( isset( $input['amount2'] ) )
	{
		$newinput['amount2'] = trim($input['amount2']);
		if(!preg_match('/^[0-9]+$/i', $newinput['amount2'])) {
			$newinput['amount2'] = '50';
		}
	}

	if( isset( $input['maxwidth'] ) )
	{
		$newinput['maxwidth'] = trim($input['maxwidth']);
		if(!preg_match('/^[0-9]+$/i', $newinput['maxwidth'])) {
			$newinput['maxwidth'] = '50';
		}
	}

	if( isset( $input['display_dashboard_capability'] ) )
	{
		global $wp_roles;
		$current_user = wp_get_current_user();
		$current_user->get_role_caps();
		$capabilities = $current_user->allcaps;
		foreach ($capabilities as $cap=>$whatever)
		{
			if( $input['display_dashboard_capability'] == $cap )
			{
				$newinput['display_dashboard_capability'] = $input['display_dashboard_capability'];
			}
		}
	}


	if (isset( $input['excludedusers'] ) )
	{
		$newinput['excludedusers'] = implode(",", $input['excludedusers']);
	}
	else
	{
		$newinput['excludedusers'] = '';
	}

	//ip exclusion list
	$target_ip_array = Array();
	if (isset( $input['excludedip'] ) )
	{
		$input_ip_array = explode(",", str_replace(" ", "", $input['excludedip'] ) );
		foreach ($input_ip_array as $ip)
		{
			if( preg_match( "/^([1-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])(\.([0-9]|[1-9][0-9]|1[0-9][0-9]|2[0-4][0-9]|25[0-5])){3}$/", $ip ) )
			{
				$target_ip_array[] = $ip;
			}
		}
	}
	$newinput['excludedip'] = implode( ",", $target_ip_array );

	if (isset( $input['logbots'] ) )
	{
		$newinput['logbots'] = trim($input['logbots']);
		if(!preg_match('/^(0|1)+$/i', $newinput['logbots'])) {
			$newinput['logbots'] = 0;
		}
	}
	else
	{
		$newinput['logbots'] = 0;
	}

	if (isset( $input['whois'] ) )
	{
		$newinput['whois'] = trim($input['whois']);
		if(!preg_match('#^http[a-zA-Z0-9-\?\.=_:;/]+$#i', $newinput['whois'])) {
			$newinput['whois'] = 'http://www.ripe.net/whois?form_type=simple&amp;searchtext=';
		}
	}

	if (isset( $input['enableip'] ) )
	{
		$newinput['enableip'] = trim($input['enableip']);
		if(!preg_match('/^(0|1)+$/i', $newinput['enableip'])) {
			$newinput['enableip'] = 0;
		}
	}
	else
	{
		$newinput['enableip'] = 0;
	}

	if( isset($input['flushdb'] ) )
	{
		if( $input['flushdb'] == "wantflushdb")
		{
			counterize_flush();
			$_SESSION["counterize_flusheddb"] = "flushed";
		}
	}
	$newinput['flushdb'] = '';

	if( isset($input['refreshua'] ) )
	{
		if( $input['refreshua'] == "wantrefreshua")
		{
			counterize_update_all_userAgents();
			$_SESSION["counterize_refreshedua"] = "refreshed";
		}
	}
	$newinput['refreshua'] = '';

	if( isset($input['deletebots'] ) )
	{
		if( $input['deletebots'] == "wantdeletebots")
		{
			counterize_delete_bots();
			$_SESSION["counterize_deletedbots"] = "deleted";
		}
	}
	$newinput['deletebots'] = '';


	return $newinput;
}








// flushes the db - be careful
function counterize_flush()
{
	global $wpdb;
	$sql = 'DELETE FROM ' . counterize_logTable();
	$num = $wpdb->query($sql);

	$sql = 'DELETE FROM ' . counterize_pageTable();
	$num = $wpdb->query($sql);

	$sql = 'DELETE FROM ' . counterize_agentsTable();
	$num = $wpdb->query($sql);

	$sql = 'DELETE FROM ' . counterize_refererTable();
	$num = $wpdb->query($sql);

	$sql = 'DELETE FROM ' . counterize_keywordTable();
	$num = $wpdb->query($sql);
}


function counterize_update_all_userAgents()
{
	global $wpdb;
	$agents = $wpdb->get_results("SELECT agentID, name FROM `" . counterize_agentsTable() . "`");
	foreach ($agents as $agent)
	{
		list(
			$browser_name, $browser_code, $browser_ver, $browser_url,
			$os_name, $os_code, $os_ver, $os_url,
			$pda_name, $pda_code, $pda_ver, $pda_url
		) = counterize_detect_browser($agent->name);

		$query = $wpdb->prepare( "UPDATE `" . counterize_agentsTable() . "` SET "
			. " browserName    = '%s', "
			. " browserCode    = '%s', "
			. " browserVersion = '%s', "
			. " browserURL     = '%s', "
			. " osName         = '%s', "
			. " osCode         = '%s', "
			. " osVersion      = '%s', "
			. " osURL          = '%s' "
			. " WHERE agentID=%d",
			$browser_name,
			$browser_code,
			$browser_ver,
			$browser_url,
			$os_name,
			$os_code,
			$os_ver,
			$os_url,
			$agent->agentID );

		$wpdb->query($query);
	}
}

//Delete the bots from the database. There is probably a more efficient way to do this.
function counterize_delete_bots()
{
	global $wpdb;
	global $counterize_plugin_dir;

	//load the bot array
	$botarray = Array();
	if (file_exists($counterize_plugin_dir . "/botlist.txt"))
	{
		$botarray = file($counterize_plugin_dir . "/botlist.txt", FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
	}

	//load the user-agents
	$agents = $wpdb->get_results("SELECT agentID, name FROM `" . counterize_agentsTable() . "`");

	//for each agent, search if it contains a bot string
	foreach ($agents as $agent)
	{

		// Run through bot-array and see if there's anything we don't like...
		foreach ($botarray as $entry)
		{
			//if we found a bot then
			if (stristr($agent->name, $entry))
			{
				//delete it!
				$query = $wpdb->prepare("DELETE FROM `" . counterize_agentsTable() . "` WHERE agentID=%d", $agent->agentID );
				$num = $wpdb->query($query);

				$query = $wpdb->prepare("DELETE FROM `" . counterize_logTable() . "` WHERE agentID=%d", $agent->agentID );
				$num = $wpdb->query($query);
			}
		}
	}
}

// deletes a entry from the database
function counterize_killEntry($entryID)
{
	global $wpdb;
	$entries = counterize_getentries(1, $entryID);
	foreach($entries as $entry)
	{
		$sql = "DELETE FROM " . counterize_logTable() . " WHERE ID={$entry->id}";
		$num = $wpdb->query($sql);

		$sql = "UPDATE ".counterize_pageTable() . " SET count = count - 1 WHERE pageID={$entry->pageID}";
		$num = $wpdb->query($sql);

		$sql = "UPDATE ".counterize_refererTable() . " SET count = count - 1 WHERE refererID={$entry->refererID}";
		$num = $wpdb->query($sql);

		$sql = "UPDATE ".counterize_agentsTable() . " SET count = count - 1 WHERE agentID={$entry->agentID}";
		$num = $wpdb->query($sql);

		$sql = "UPDATE ".counterize_keywordTable() . " SET count = count - 1 WHERE keywordID={$entry->keywordID}";
		$num = $wpdb->query($sql);
	}
}


?>